/*============================================================================
 * fifo.h
 *      Header file for implementing software FIFO.
 *==========================================================================*/

#ifndef _fifo_h_
#define _fifo_h_

#include <pthread.h>

typedef int     fifoIntType;

typedef struct
{
    void                *data;
    fifoIntType         frameSize;
    fifoIntType         N_frames;
    fifoIntType         headIdx;
    fifoIntType         tailIdx;
#ifdef __WINDOWS__
    HANDLE              write_mutex;
    HANDLE              read_mutex;
#else    
    pthread_mutex_t     write_mutex;
    pthread_mutex_t     read_mutex;
#endif    
} fifoDef;

typedef unsigned int    fifoNodeSizeType;
#define FIFO_NODE_SIZE  (sizeof(fifoNodeSizeType))

#define FIFO_PAYLOAD_SIZE(data_ptr) (*((fifoNodeSizeType *) (data_ptr) -1))

/* Macros */
#define FIFO_IS_EMPTY(f)  ((f)->headIdx == (f)->tailIdx)
#define FIFO_IS_FULL(f)   (FIFO_TAIL_NEXT(f) == (f)->headIdx)
#define FIFO_HEAD_NEXT(f) (((f)->headIdx + 1) % (f)->N_frames)
#define FIFO_TAIL_NEXT(f) (((f)->tailIdx + 1) % (f)->N_frames)

/* functions. */
int fifoCreate(fifoIntType frameSize, fifoIntType N_frames, fifoDef **fifo);
void fifoDestroy(fifoDef *fifo);
void *fifoReadPtr(fifoDef *fifo);
int fifoIsEmpty(fifoDef *fifo);
int fifoIsFull(fifoDef *fifo);
void fifoReadDone(fifoDef *fifo);
void fifoReadDoneAbort(fifoDef *fifo);
int fifoRead(fifoDef *fifo, void *data);
void *fifoWritePtr(fifoDef *fifo);
void fifoWriteDone(fifoDef *fifo);
void fifoWriteDoneAbort(fifoDef *fifo);
int fifoWrite(fifoDef *fifo, void *data);
fifoIntType fifoElemUsed(fifoDef *fifo);

#endif
