////////////////////////////////////////////////////////////////////////////
// get_wall_time.cc
//      Simple script helper program to output the current realtime clock.
//      Or, to compute a difference between two clock timestamps, and
//      return the result in seconds.  It is used to allow the lag of
//      audio and video capture to be fixed when streaming AR scenery
//      and associated audio to files.
//
// Author: Jon W. Wallace, Lafayette College
// Date:   May 2020
////////////////////////////////////////////////////////////////////////////

#include <time.h>
#include <stdio.h>

#define CONST_1E9       1000000000

int main(int argc, char *argv[])
{
    struct timespec t_start;
    struct timespec t_diff;
    struct timespec t;

    if (argc < 2)
    {
        fprintf(stderr, "usage: %s n\n", argv[0]);
        fprintf(stderr, "               Prints real time clock (secs, nsecs)\n");
        fprintf(stderr, "       %s d [sec1] [nsec1] [sec2] [nsec2]\n", argv[0]);
        fprintf(stderr, "               Prints difference (t2-t1) of two times\n");
        goto backout_base;
    }

    if (argv[1][0] == 'n')
    {
        // Get current timestamp
        clock_gettime(CLOCK_REALTIME, &t);

        fprintf(stdout, "%ld %ld\n", t.tv_sec, t.tv_nsec);
    }
    else if (argv[1][0] == 'd')
    {
        if (argc < 6)
        {
            fprintf(stderr, "Invalid number of inputs for time stamp difference.\n");
            goto backout_base;
        }

        if ((sscanf(argv[2], "%ld", &t_start.tv_sec) != 1) ||
            (sscanf(argv[3], "%ld", &t_start.tv_nsec) != 1) ||
            (sscanf(argv[4], "%ld", &t.tv_sec) != 1) ||
            (sscanf(argv[5], "%ld", &t.tv_nsec) != 1))
        {
            fprintf(stderr, "Invalid time stamp input.\n");
            goto backout_base;
        }
        
        t_diff.tv_sec = (t.tv_sec - t_start.tv_sec);
        t_diff.tv_nsec = (t.tv_nsec - t_start.tv_nsec);
        if (t_diff.tv_nsec < 0)
        {
            t_diff.tv_nsec += CONST_1E9;
            t_diff.tv_sec--;
        }
        else if (t_diff.tv_nsec >= CONST_1E9)
        {
            t_diff.tv_nsec -= CONST_1E9;
            t_diff.tv_sec++;
        }

        // Output in s
        fprintf(stdout, "%.9lf\n", ((double)t_diff.tv_sec + t_diff.tv_nsec/(double)(CONST_1E9)));
    }

    return(0);
    
 backout_base:
    return(-1);
}


