/*============================================================================
 * arProfile.h
 *      Header file for profiling code.
 *==========================================================================*/

#ifndef _arProfile_h_
#define _arProfile_h_

#include <time.h>

// Structure to hold profiling information.  Holds CPU time.
typedef unsigned long           io_int;
typedef struct
{
    /* Counts for CPU time */
    /*clock_t     start;
    clock_t     end;
    clock_t     total; */
#ifdef __WINDOWS__
    DWORD start;
    DWORD end;
    DWORD total;
#else    
    struct timespec start;
    struct timespec end;
    struct timespec total;
#endif
    io_int      io_count;
} cpuProfDef;

// Structure to hold profiling information for wall clock time
typedef struct
{
    /* Counts for wall-clock time */
#ifdef __WINDOWS__
    DWORD start;
    DWORD end;
    DWORD total;
#else    
    struct timespec start;
    struct timespec end;
    struct timespec total;
#endif    
} wallProfDef;

void profCpuInit(cpuProfDef *p);
void profWallInit(wallProfDef *p);
void profCpuStart(cpuProfDef *p);
void profWallStart(wallProfDef *p);
void profCpuEnd(cpuProfDef *p, io_int count);
void profWallEnd(wallProfDef *p);
double profTimeSec(cpuProfDef *p);
double profTimeSec(wallProfDef *p);

void profPrintOne(const char *name, cpuProfDef *time1, cpuProfDef *timeAll, int numFrames);
void profPrintOne(const char *name, wallProfDef *time1, wallProfDef *timeAll, int numFrames);

void profSampleThreadStart();
void profSampleThreadStop();
void profFifoPrint();

double profWallTime();

#endif
